@@ -38,11 +38,21 @@ typedef struct tray_output_t {
     TAILQ_ENTRY(tray_output_t) tray_outputs;
 } tray_output_t;
 
+/* Matches i3/include/data.h */
+struct Rect {
+    uint32_t x;
+    uint32_t y;
+    uint32_t width;
+    uint32_t height;
+};
+
 typedef struct config_t {
     uint32_t modifier;
     TAILQ_HEAD(bindings_head, binding_t) bindings;
     position_t position;
     bool verbose;
+    uint32_t bar_height;
+    struct Rect padding;
     bool transparency;
     struct xcb_color_strings_t colors;
     bool disable_binding_mode_indicator;
