@@ -59,7 +59,7 @@ xcb_visualtype_t *visual_type;
 uint8_t depth;
 xcb_colormap_t colormap;
 
-/* Overall height of the bar (based on font size) */
+/* Overall height of the bar */
 int bar_height;
 
 /* These are only relevant for XKB, which we only need for grabbing modifiers */
@@ -180,7 +180,7 @@ static void draw_separator(i3_output *output, uint32_t x, struct status_block *b
         /* Draw a custom separator. */
         uint32_t separator_x = MAX(x - block->sep_block_width, center_x - separator_symbol_width / 2);
         draw_util_text(config.separator_symbol, &output->statusline_buffer, sep_fg, bar_bg,
-                       separator_x, logical_px(ws_voff_px), x - separator_x);
+                       separator_x, bar_height / 2 - font.height / 2, x - separator_x);
     }
 }
 
@@ -203,7 +203,7 @@ static uint32_t predict_statusline_length(bool use_short_text) {
         if (block->border)
             render->width += logical_px(block->border_left + block->border_right);
 
-        /* Compute offset and append for text aligment in min_width. */
+        /* Compute offset and append for text alignment in min_width. */
         if (block->min_width <= render->width) {
             render->x_offset = 0;
             render->x_append = 0;
@@ -1354,8 +1354,27 @@ void init_xcb_late(char *fontname) {
     /* Load the font */
     font = load_font(fontname, true);
     set_font(&font);
-    DLOG("Calculated font height: %d\n", font.height);
-    bar_height = font.height + 2 * logical_px(ws_voff_px);
+    DLOG("Calculated font-height: %d\n", font.height);
+
+    /*
+     * If the bar height was explicitly set (but padding was not set), use
+     * it. Otherwise, calculate it based on the font size.
+     */
+    const int default_px = font.height + 2 * logical_px(ws_voff_px);
+    int padding_scaled =
+        logical_px(config.padding.y) +
+        logical_px(config.padding.height);
+    if (config.bar_height > 0 &&
+        config.padding.x == 0 &&
+        config.padding.y == 0 &&
+        config.padding.width == 0 &&
+        config.padding.height == 0) {
+        padding_scaled = config.bar_height - default_px;
+        DLOG("setting padding_scaled=%d based on bar_height=%d\n", padding_scaled, config.bar_height);
+    } else {
+        DLOG("padding: x=%d, y=%d -> padding_scaled=%d\n", config.padding.x, config.padding.y, padding_scaled);
+    }
+    bar_height = default_px + padding_scaled;
     icon_size = bar_height - 2 * logical_px(config.tray_padding);
 
     if (config.separator_symbol)
@@ -1973,7 +1992,7 @@ void reconfig_windows(bool redraw_bars) {
  */
 static void draw_button(surface_t *surface, color_t fg_color, color_t bg_color, color_t border_color,
                         int x, int width, int text_width, i3String *text) {
-    int height = font.height + 2 * logical_px(ws_voff_px) - 2 * logical_px(1);
+    int height = bar_height - 2 * logical_px(1);
 
     /* Draw the border of the button. */
     draw_util_rectangle(surface, border_color, x, logical_px(1), width, height);
@@ -1983,7 +2002,7 @@ static void draw_button(surface_t *surface, color_t fg_color, color_t bg_color,
                         width - 2 * logical_px(1), height - 2 * logical_px(1));
 
     draw_util_text(text, surface, fg_color, bg_color, x + (width - text_width) / 2,
-                   logical_px(ws_voff_px), text_width);
+                   bar_height / 2 - font.height / 2, text_width);
 }
 
 /*
@@ -1998,7 +2017,7 @@ void draw_bars(bool unhide) {
 
     i3_output *outputs_walk;
     SLIST_FOREACH (outputs_walk, outputs, slist) {
-        int workspace_width = 0;
+        int workspace_width = logical_px(config.padding.x);
 
         if (!outputs_walk->active) {
             DLOG("Output %s inactive, skipping...\n", outputs_walk->name);
@@ -2082,6 +2101,7 @@ void draw_bars(bool unhide) {
 
             int16_t visible_statusline_width = MIN(statusline_width, max_statusline_width);
             int x_dest = outputs_walk->rect.w - tray_width - logical_px((tray_width > 0) * sb_hoff_px) - visible_statusline_width;
+            x_dest -= logical_px(config.padding.width);
 
             draw_statusline(outputs_walk, clip_left, use_focus_colors, use_short_text);
             draw_util_copy_surface(&outputs_walk->statusline_buffer, &outputs_walk->buffer, 0, 0,
