@@ -3,6 +3,7 @@
 xmacro(_NET_WM_USER_TIME) \
 xmacro(_NET_STARTUP_ID) \
 xmacro(_NET_WORKAREA) \
+xmacro(_NET_WM_ICON) \
 xmacro(WM_PROTOCOLS) \
 xmacro(WM_DELETE_WINDOW) \
 xmacro(UTF8_STRING) \
@@ -15,8 +16,10 @@ xmacro(I3_CONFIG_PATH) \
 xmacro(I3_SYNC) \
 xmacro(I3_SHMLOG_PATH) \
 xmacro(I3_PID) \
+xmacro(I3_LOG_STREAM_SOCKET_PATH) \
 xmacro(I3_FLOATING_WINDOW) \
 xmacro(_NET_REQUEST_FRAME_EXTENTS) \
 xmacro(_NET_FRAME_EXTENTS) \
 xmacro(_MOTIF_WM_HINTS) \
-xmacro(WM_CHANGE_STATE)
+xmacro(WM_CHANGE_STATE) \
+xmacro(MANAGER)
