@@ -79,16 +79,22 @@ void floating_check_size(Con *floating_con, bool prefer_height) {
     Rect floating_sane_max_dimensions;
     Con *focused_con = con_descend_focused(floating_con);
 
+    DLOG("deco_rect.height = %d\n", focused_con->deco_rect.height);
     Rect border_rect = con_border_style_rect(focused_con);
     /* We have to do the opposite calculations that render_con() do
      * to get the exact size we want. */
     border_rect.width = -border_rect.width;
-    border_rect.width += 2 * focused_con->border_width;
     border_rect.height = -border_rect.height;
+
+    /* undo x11 border */
+    border_rect.width += 2 * focused_con->border_width;
     border_rect.height += 2 * focused_con->border_width;
-    if (con_border_style(focused_con) == BS_NORMAL) {
-        border_rect.height += render_deco_height();
-    }
+
+    DLOG("floating_check_size, want min width %d, min height %d, border extra: w=%d, h=%d\n",
+         floating_sane_min_width,
+         floating_sane_min_height,
+         border_rect.width,
+         border_rect.height);
 
     i3Window *window = focused_con->window;
     if (window != NULL) {
@@ -253,7 +259,7 @@ bool floating_enable(Con *con, bool automatic) {
         }
         /* Consider the part of the focus stack of our current workspace:
          * [ ... S_{i-1} S_{i} S_{i+1} ... ]
-         * Where S_{x} is a container tree and the container 'con' that is beeing switched to
+         * Where S_{x} is a container tree and the container 'con' that is being switched to
          * floating belongs in S_{i}. The new floating container, 'nc', will have the
          * workspace as its parent so it needs to be placed in this stack. If C was focused
          * we just need to call con_focus(). Otherwise, nc must be placed before or after S_{i}.
@@ -319,9 +325,6 @@ bool floating_enable(Con *con, bool automatic) {
     x_set_name(nc, name);
     free(name);
 
-    /* find the height for the decorations */
-    int deco_height = render_deco_height();
-
     DLOG("Original rect: (%d, %d) with %d x %d\n", con->rect.x, con->rect.y, con->rect.width, con->rect.height);
     DLOG("Geometry = (%d, %d) with %d x %d\n", con->geometry.x, con->geometry.y, con->geometry.width, con->geometry.height);
     nc->rect = con->geometry;
@@ -347,19 +350,15 @@ bool floating_enable(Con *con, bool automatic) {
     con->floating = FLOATING_USER_ON;
 
     /* 4: set the border style as specified with new_float */
-    if (automatic)
-        con->border_style = config.default_floating_border;
+    if (automatic) {
+        con->border_style = con->max_user_border_style = config.default_floating_border;
+    }
 
     /* Add pixels for the decoration. */
-    Rect border_style_rect = con_border_style_rect(con);
+    Rect bsr = con_border_style_rect(con);
 
-    nc->rect.height -= border_style_rect.height;
-    nc->rect.width -= border_style_rect.width;
-
-    /* Add some more pixels for the title bar */
-    if (con_border_style(con) == BS_NORMAL) {
-        nc->rect.height += deco_height;
-    }
+    nc->rect.height -= bsr.height;
+    nc->rect.width -= bsr.width;
 
     /* Honor the X11 border */
     nc->rect.height += con->border_width * 2;
@@ -405,14 +404,6 @@ bool floating_enable(Con *con, bool automatic) {
 
     DLOG("Floating rect: (%d, %d) with %d x %d\n", nc->rect.x, nc->rect.y, nc->rect.width, nc->rect.height);
 
-    /* 5: Subtract the deco_height in order to make the floating window appear
-     * at precisely the position it specified in its original geometry (which
-     * is what applications might remember). */
-    deco_height = (con->border_style == BS_NORMAL ? render_deco_height() : 0);
-    nc->rect.y -= deco_height;
-
-    DLOG("Corrected y = %d (deco_height = %d)\n", nc->rect.y, deco_height);
-
     /* render the cons to get initial window_rect correct */
     render_con(nc);
 
@@ -698,6 +689,10 @@ void floating_resize_window(Con *con, const bool proportional,
                             const xcb_button_press_event_t *event) {
     DLOG("floating_resize_window\n");
 
+    /* Push changes before resizing, so that the window gets raised now and not
+     * after the user releases the mouse button */
+    tree_render();
+
     /* corner saves the nearest corner to the original click. It contains
      * a bitmask of the nearest borders (BORDER_LEFT, BORDER_RIGHT, …) */
     border_t corner = 0;
