@@ -70,11 +70,9 @@ bool event_is_ignored(const int sequence, const int response_type) {
             event->response_type != response_type)
             continue;
 
-        /* instead of removing a sequence number we better wait until it gets
-         * garbage collected. it may generate multiple events (there are multiple
-         * enter_notifies for one configure_request, for example). */
-        //SLIST_REMOVE(&ignore_events, event, Ignore_Event, ignore_events);
-        //free(event);
+        /* Instead of removing & freeing a sequence number we better wait until
+         * it gets garbage collected. It may generate multiple events (there
+         * are multiple enter_notifies for one configure_request, for example). */
         return true;
     }
 
@@ -216,18 +214,30 @@ static void handle_motion_notify(xcb_motion_notify_event_t *event) {
         return;
 
     /* see over which rect the user is */
-    Con *current;
-    TAILQ_FOREACH_REVERSE (current, &(con->nodes_head), nodes_head, nodes) {
-        if (!rect_contains(current->deco_rect, event->event_x, event->event_y))
-            continue;
+    if (con->window != NULL) {
+        if (rect_contains(con->deco_rect, event->event_x, event->event_y)) {
+            /* We found the rect, let’s see if this window is focused */
+            if (TAILQ_FIRST(&(con->parent->focus_head)) == con)
+                return;
 
-        /* We found the rect, let’s see if this window is focused */
-        if (TAILQ_FIRST(&(con->focus_head)) == current)
+            con_focus(con);
+            x_push_changes(croot);
             return;
+        }
+    } else {
+        Con *current;
+        TAILQ_FOREACH_REVERSE (current, &(con->nodes_head), nodes_head, nodes) {
+            if (!rect_contains(current->deco_rect, event->event_x, event->event_y))
+                continue;
 
-        con_focus(current);
-        x_push_changes(croot);
-        return;
+            /* We found the rect, let’s see if this window is focused */
+            if (TAILQ_FIRST(&(con->focus_head)) == current)
+                return;
+
+            con_focus(current);
+            x_push_changes(croot);
+            return;
+        }
     }
 }
 
@@ -270,7 +280,7 @@ static void handle_map_request(xcb_map_request_event_t *event) {
  * Configure requests are received when the application wants to resize windows
  * on their own.
  *
- * We generate a synthethic configure notify event to signalize the client its
+ * We generate a synthetic configure notify event to signalize the client its
  * "new" position.
  *
  */
@@ -320,15 +330,9 @@ static void handle_configure_request(xcb_configure_request_event_t *event) {
     Con *fullscreen = con_get_fullscreen_covering_ws(workspace);
 
     if (fullscreen != con && con_is_floating(con) && con_is_leaf(con)) {
-        /* find the height for the decorations */
-        int deco_height = con->deco_rect.height;
         /* we actually need to apply the size/position changes to the *parent*
          * container */
         Rect bsr = con_border_style_rect(con);
-        if (con->border_style == BS_NORMAL) {
-            bsr.y += deco_height;
-            bsr.height -= deco_height;
-        }
         Con *floatingcon = con->parent;
         Rect newrect = floatingcon->rect;
 
@@ -801,14 +805,14 @@ static void handle_client_message(xcb_client_message_event_t *event) {
     } else if (event->type == A_WM_CHANGE_STATE) {
         /* http://tronche.com/gui/x/icccm/sec-4.html#s-4.1.4 */
         if (event->data.data32[0] == XCB_ICCCM_WM_STATE_ICONIC) {
-            /* For compatiblity reasons, Wine will request iconic state and cannot ensure that the WM has agreed on it;
+            /* For compatibility reasons, Wine will request iconic state and cannot ensure that the WM has agreed on it;
              * immediately revert to normal to avoid being stuck in a paused state. */
-            DLOG("Client has requested iconic state, rejecting. (window = %d)\n", event->window);
+            DLOG("Client has requested iconic state, rejecting. (window = %08x)\n", event->window);
             long data[] = {XCB_ICCCM_WM_STATE_NORMAL, XCB_NONE};
             xcb_change_property(conn, XCB_PROP_MODE_REPLACE, event->window,
                                 A_WM_STATE, A_WM_STATE, 32, 2, data);
         } else {
-            DLOG("Not handling WM_CHANGE_STATE request. (window = %d, state = %d)\n", event->window, event->data.data32[0]);
+            DLOG("Not handling WM_CHANGE_STATE request. (window = %08x, state = %d)\n", event->window, event->data.data32[0]);
         }
     } else if (event->type == A__NET_CURRENT_DESKTOP) {
         /* This request is used by pagers and bars to change the current
@@ -876,7 +880,7 @@ static void handle_client_message(xcb_client_message_event_t *event) {
             tree_close_internal(con, KILL_WINDOW, false);
             tree_render();
         } else {
-            DLOG("Couldn't find con for _NET_CLOSE_WINDOW request. (window = %d)\n", event->window);
+            DLOG("Couldn't find con for _NET_CLOSE_WINDOW request. (window = %08x)\n", event->window);
         }
     } else if (event->type == A__NET_WM_MOVERESIZE) {
         /*
@@ -885,7 +889,7 @@ static void handle_client_message(xcb_client_message_event_t *event) {
          */
         Con *con = con_by_window_id(event->window);
         if (!con || !con_is_floating(con)) {
-            DLOG("Couldn't find con for _NET_WM_MOVERESIZE request, or con not floating (window = %d)\n", event->window);
+            DLOG("Couldn't find con for _NET_WM_MOVERESIZE request, or con not floating (window = %08x)\n", event->window);
             return;
         }
         DLOG("Handling _NET_WM_MOVERESIZE request (con = %p)\n", con);
@@ -1058,6 +1062,76 @@ static void handle_focus_in(xcb_focus_in_event_t *event) {
     tree_render();
 }
 
+/*
+ * Log FocusOut events.
+ *
+ */
+static void handle_focus_out(xcb_focus_in_event_t *event) {
+    Con *con = con_by_window_id(event->event);
+    const char *window_name, *mode, *detail;
+
+    if (con != NULL) {
+        window_name = con->name;
+        if (window_name == NULL) {
+            window_name = "<unnamed con>";
+        }
+    } else if (event->event == root) {
+        window_name = "<the root window>";
+    } else {
+        window_name = "<unknown window>";
+    }
+
+    switch (event->mode) {
+        case XCB_NOTIFY_MODE_NORMAL:
+            mode = "Normal";
+            break;
+        case XCB_NOTIFY_MODE_GRAB:
+            mode = "Grab";
+            break;
+        case XCB_NOTIFY_MODE_UNGRAB:
+            mode = "Ungrab";
+            break;
+        case XCB_NOTIFY_MODE_WHILE_GRABBED:
+            mode = "WhileGrabbed";
+            break;
+        default:
+            mode = "<unknown>";
+            break;
+    }
+
+    switch (event->detail) {
+        case XCB_NOTIFY_DETAIL_ANCESTOR:
+            detail = "Ancestor";
+            break;
+        case XCB_NOTIFY_DETAIL_VIRTUAL:
+            detail = "Virtual";
+            break;
+        case XCB_NOTIFY_DETAIL_INFERIOR:
+            detail = "Inferior";
+            break;
+        case XCB_NOTIFY_DETAIL_NONLINEAR:
+            detail = "Nonlinear";
+            break;
+        case XCB_NOTIFY_DETAIL_NONLINEAR_VIRTUAL:
+            detail = "NonlinearVirtual";
+            break;
+        case XCB_NOTIFY_DETAIL_POINTER:
+            detail = "Pointer";
+            break;
+        case XCB_NOTIFY_DETAIL_POINTER_ROOT:
+            detail = "PointerRoot";
+            break;
+        case XCB_NOTIFY_DETAIL_NONE:
+            detail = "NONE";
+            break;
+        default:
+            detail = "unknown";
+            break;
+    }
+
+    DLOG("focus change out: window 0x%08x (con %p, %s) lost focus with detail=%s, mode=%s\n", event->event, con, window_name, detail, mode);
+}
+
 /*
  * Handles ConfigureNotify events for the root window, which are generated when
  * the monitor configuration changed.
@@ -1074,6 +1148,23 @@ static void handle_configure_notify(xcb_configure_notify_event_t *event) {
         return;
     }
     randr_query_outputs();
+
+    ipc_send_event("output", I3_IPC_EVENT_OUTPUT, "{\"change\":\"unspecified\"}");
+}
+
+/*
+ * Handles SelectionClear events for the root window, which are generated when
+ * we lose ownership of a selection.
+ */
+static void handle_selection_clear(xcb_selection_clear_event_t *event) {
+    if (event->selection != wm_sn) {
+        DLOG("SelectionClear for unknown selection %d, ignoring\n", event->selection);
+        return;
+    }
+    LOG("Lost WM_Sn selection, exiting.\n");
+    exit(EXIT_SUCCESS);
+
+    /* unreachable */
 }
 
 /*
@@ -1087,14 +1178,24 @@ static bool handle_class_change(Con *con, xcb_get_property_reply_t *prop) {
 }
 
 /*
- * Handles the _MOTIF_WM_HINTS property of specifing window deocration settings.
+ * Handles the WM_CLIENT_MACHINE property for assignments and criteria selection.
+ *
+ */
+static bool handle_machine_change(Con *con, xcb_get_property_reply_t *prop) {
+    window_update_machine(con->window, prop);
+    con = remanage_window(con);
+    return true;
+}
+
+/*
+ * Handles the _MOTIF_WM_HINTS property of specifying window deocration settings.
  *
  */
 static bool handle_motif_hints_change(Con *con, xcb_get_property_reply_t *prop) {
     border_style_t motif_border_style;
-    window_update_motif_hints(con->window, prop, &motif_border_style);
+    bool has_mwm_hints = window_update_motif_hints(con->window, prop, &motif_border_style);
 
-    if (motif_border_style != con->border_style && motif_border_style != BS_NORMAL) {
+    if (has_mwm_hints && motif_border_style != con->border_style) {
         DLOG("Update border style of con %p to %d\n", con, motif_border_style);
         con_set_border_style(con, motif_border_style, con->current_border_width);
 
@@ -1175,6 +1276,14 @@ static bool handle_i3_floating(Con *con, xcb_get_property_reply_t *prop) {
     return true;
 }
 
+static bool handle_windowicon_change(Con *con, xcb_get_property_reply_t *prop) {
+    window_update_icon(con->window, prop);
+
+    x_push_changes(croot);
+
+    return true;
+}
+
 /* Returns false if the event could not be processed (e.g. the window could not
  * be found), true otherwise */
 typedef bool (*cb_property_handler_t)(Con *con, xcb_get_property_reply_t *property);
@@ -1197,7 +1306,9 @@ static struct property_handler_t property_handlers[] = {
     {0, UINT_MAX, handle_strut_partial_change},
     {0, UINT_MAX, handle_window_type},
     {0, UINT_MAX, handle_i3_floating},
-    {0, 5 * sizeof(uint64_t), handle_motif_hints_change}};
+    {0, 128, handle_machine_change},
+    {0, 5 * sizeof(uint64_t), handle_motif_hints_change},
+    {0, UINT_MAX, handle_windowicon_change}};
 #define NUM_HANDLERS (sizeof(property_handlers) / sizeof(struct property_handler_t))
 
 /*
@@ -1219,7 +1330,9 @@ void property_handlers_init(void) {
     property_handlers[8].atom = A__NET_WM_STRUT_PARTIAL;
     property_handlers[9].atom = A__NET_WM_WINDOW_TYPE;
     property_handlers[10].atom = A_I3_FLOATING_WINDOW;
-    property_handlers[11].atom = A__MOTIF_WM_HINTS;
+    property_handlers[11].atom = XCB_ATOM_WM_CLIENT_MACHINE;
+    property_handlers[12].atom = A__MOTIF_WM_HINTS;
+    property_handlers[13].atom = A__NET_WM_ICON;
 }
 
 static void property_notify(uint8_t state, xcb_window_t window, xcb_atom_t atom) {
@@ -1237,7 +1350,7 @@ static void property_notify(uint8_t state, xcb_window_t window, xcb_atom_t atom)
     }
 
     if (handler == NULL) {
-        //DLOG("Unhandled property notify for atom %d (0x%08x)\n", atom, atom);
+        /* DLOG("Unhandled property notify for atom %d (0x%08x)\n", atom, atom); */
         return;
     }
 
@@ -1395,6 +1508,10 @@ void handle_event(int type, xcb_generic_event_t *event) {
             handle_focus_in((xcb_focus_in_event_t *)event);
             break;
 
+        case XCB_FOCUS_OUT:
+            handle_focus_out((xcb_focus_out_event_t *)event);
+            break;
+
         case XCB_PROPERTY_NOTIFY: {
             xcb_property_notify_event_t *e = (xcb_property_notify_event_t *)event;
             last_timestamp = e->time;
@@ -1406,8 +1523,12 @@ void handle_event(int type, xcb_generic_event_t *event) {
             handle_configure_notify((xcb_configure_notify_event_t *)event);
             break;
 
+        case XCB_SELECTION_CLEAR:
+            handle_selection_clear((xcb_selection_clear_event_t *)event);
+            break;
+
         default:
-            //DLOG("Unhandled event of type %d\n", type);
+            /* DLOG("Unhandled event of type %d\n", type); */
             break;
     }
 }
