@@ -85,6 +85,10 @@ Con *get_assigned_output(const char *name, long parsed_num) {
     Con *output = NULL;
     struct Workspace_Assignment *assignment;
     TAILQ_FOREACH (assignment, &ws_assignments, ws_assignments) {
+        if (assignment->output == NULL) {
+            continue;
+        }
+
         if (name && strcmp(assignment->name, name) == 0) {
             DLOG("Found workspace name=\"%s\" assignment to output \"%s\"\n",
                  name, assignment->output);
@@ -156,6 +160,7 @@ Con *workspace_get(const char *num) {
     workspace->workspace_layout = config.default_layout;
     workspace->num = parsed_num;
     workspace->type = CT_WORKSPACE;
+    workspace->gaps = gaps_for_workspace(workspace);
 
     con_attach(workspace, output_get_content(output), false);
     _workspace_apply_default_orientation(workspace);
@@ -281,6 +286,7 @@ Con *create_workspace_on_output(Output *output, Con *content) {
         ws->num = c;
         sasprintf(&(ws->name), "%d", c);
     }
+
     con_attach(ws, content, false);
 
     char *name;
@@ -288,6 +294,8 @@ Con *create_workspace_on_output(Output *output, Con *content) {
     x_set_name(ws, name);
     free(name);
 
+    ws->gaps = gaps_for_workspace(ws);
+
     ws->fullscreen_mode = CF_OUTPUT;
 
     ws->workspace_layout = config.default_layout;
@@ -299,16 +307,16 @@ Con *create_workspace_on_output(Output *output, Con *content) {
 
 /*
  * Returns true if the workspace is currently visible. Especially important for
- * multi-monitor environments, as they can have multiple currenlty active
+ * multi-monitor environments, as they can have multiple currently active
  * workspaces.
  *
  */
 bool workspace_is_visible(Con *ws) {
     Con *output = con_get_output(ws);
-    if (output == NULL)
+    if (output == NULL) {
         return false;
+    }
     Con *fs = con_get_fullscreen_con(output, CF_OUTPUT);
-    LOG("workspace visible? fs = %p, ws = %p\n", fs, ws);
     return (fs == ws);
 }
 
